// -*-c++-*-
#ifndef _DEBUGMEM_H_
#define _DEBUGMEM_H_
// Debug memory allocator -- catches leaks, dual frees, writes past end
//
// Copyright (c) David Drysdale 2001, 2010
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
// AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of the author shall not be
// used in advertising or otherwise to promote the sale, use or other dealings
// in this Software without prior written authorization from the author.
// ----------------------------------------------------------------
//
// To use these functions, replace 'new Class()' with 'NEW Class' and
// 'delete ptr;' with 'DELETE(ptr)'
// In a non-debug build, these will fall through to the normal new/delete.
// In a debug build, the tracking versions are used.
//
// If you are using these macros in multi-threaded code, you should
// #define MT_DEBUGMEM before including this file (and in the compilation of
// debugmem.cpp).

#ifdef DEBUG

// Placement-new (and matching placement-delete) including code location
void *operator new(size_t, const char*, int) throw (std::bad_alloc);
void operator delete(void *, const char*, int) throw ();

// Internal implementation functions
void* debug_new(size_t size, const char *file, int line) throw (std::bad_alloc);
void debug_delete(const void *, const char*, int) throw ();

// Map NEW through to placement new including location information
#define NEW new(__FILE__,__LINE__)

// Map DELETE through to:
// a) explicit call to destructor
// b) explicit call to placement delete including location information
template <class T> void p_delete(const T *p, const char* file, int line) throw ()
{
  // Static type of the variable is T
  p->~T();
  debug_delete(p, file, line);
}
#define DELETE(p) p_delete(p,__FILE__,__LINE__)

#else
// Fall through to normal keywords
#define NEW new
#define DELETE(p) delete p
#endif

#endif
