// -*-c++-*-
#ifndef _GETOPT_LOCAL_H
#define _GETOPT_LOCAL_H
// Command line argument processing (compatible with UNIX getopt style)
//
// Copyright (c) 2005-2006 David Drysdale
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
// AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of the author shall not be
// used in advertising or otherwise to promote the sale, use or other dealings
// in this Software without prior written authorization from the author.

#include <cstdio>

//! Argument value
static const char* optarg_l = 0;
//! Argument to process next
static int optind_l = 1;

//! Process next command line argument.
//! @param[in] argc Number of arguments
//! @param[in] argv Array of arguments
//! @param[in] options Description of available command line options
//! @return Current argument or -1 if no more arguments.
static int getopt_l(int argc, char *argv[], const char* options)
{
  if (optind_l >= argc) return -1;
  if (argv[optind_l][0] != '-') return -1;

  char option = argv[optind_l][1];
  optind_l++;

  // Check whether this is an allowed option
  const char *p = options;
  while ((*p != '\0') && (*p != option)) p++;
  if (*p == '\0') return -1;

  // Check whether this option expects an argument
  if (*(p+1) == ':') {
    if (optind_l >= argc) return -1;
    optarg_l = argv[optind_l];
    optind_l++;
  }
  return option;
}

#endif
