// Test program for C++ locking utility
//
// Copyright (c) David Drysdale 2000
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
// AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
// AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
// CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
// Except as contained in this notice, the name of the author shall not be
// used in advertising or otherwise to promote the sale, use or other dealings
// in this Software without prior written authorization from the author.

#define DEBUG
#include <lock.h>
#define TRCCLASSNAME "testlock"
#include <cpptrace.h>

#include <pthread.h>

DECLARE_LOCK(sLockA);
DECLARE_LOCK(sLockB);
DECLARE_LOCK(sLockC);
DECLARE_LOCK(sLockD);
DECLARE_LOCK(sLockE);
DECLARE_LOCK(sLockF);
DECLARE_LOCK(sLockG);
DECLARE_LOCK(sLockH);
DECLARE_LOCK(sLockI);

extern "C" void *thread2_fn(void *);
void *thread2_fn(void *p)
{
  // generate some errors
  LOCK(sLockA);
  LOCK(sLockB);
  UNLOCK(sLockA); // error - not last locked
  UNLOCK(sLockB);

  UNLOCK(sLockB); // error - negative lock count
  return(p);
}


int main()
{
  TRACE_FN("main");
  // build a locking hierarchy
  LOCK(sLockA); UNLOCK(sLockA);
  {BLOCK_LOCK(sLockB);}
  LOCK(sLockC); UNLOCK(sLockC);
  LOCK(sLockA); LOCK(sLockB); UNLOCK(sLockB); UNLOCK(sLockA);
  LOCK(sLockB); LOCK(sLockC); UNLOCK(sLockC); UNLOCK(sLockB);
  LOCK(sLockB); LOCK(sLockD); UNLOCK(sLockD); UNLOCK(sLockB);

  // repeat should not turn up in hierarchy separately
  LOCK(sLockA); LOCK(sLockB); UNLOCK(sLockB); UNLOCK(sLockA);

  // Generate a deadlock
  LOCK(sLockE); LOCK(sLockF); UNLOCK(sLockF); UNLOCK(sLockE);
  LOCK(sLockF); LOCK(sLockE); UNLOCK(sLockE); UNLOCK(sLockF);

  // Generate a 3way deadlock
  LOCK(sLockG); LOCK(sLockH); UNLOCK(sLockH); UNLOCK(sLockG);
  LOCK(sLockH); LOCK(sLockI); UNLOCK(sLockI); UNLOCK(sLockH);
  LOCK(sLockI); LOCK(sLockG); UNLOCK(sLockG); UNLOCK(sLockI);

  pthread_t thread;
  int status;
  status = pthread_create(&thread, NULL, thread2_fn, NULL);
  TRC_NRM("status is "<< status);
  void *result;
  status = pthread_join(thread, &result);
  TRC_NRM("status is "<< status);

  return 0;
}
